# Summary

* [Introduction](README.md)
  * [Introduction to OAuth 2.0 and OpenID Connect](README.md#introduction-to-oauth-20-and-openid-connect)
  * [Introduction to Hydra](README.md#introduction-to-hydra)
  * [OAuth2 Case Study](README.md#oauth-20-case-study)
* [5 Minute Tutorial](tutorial.md)
* [Using Hydra](install.md)
  * [Installing Hydra](install.md#installing-hydra)
  * [Configuring Hydra](install.md#configuring-hydra)
  * [Running Hydra](install.md#running-hydra)
* Understanding Hydra
  * [OAuth2 & OpenID Connect](oauth2.md)
    * [Overview](oauth2.md#overview)
    * [Confirming User Consent: Consent App Flow](oauth2.md#consent-app-flow)
    * [Validating Tokens: OAuth2 Token Introspection](oauth2.md#oauth2-token-introspection)
    * [OAuth2 Scopes](oauth2.md#oauth2-scopes)
  * [JSON Web Keys](jwk.md)
  * [Access Control Policies](access-control.md)
    * [Introduction](access-control.md#introduction-to-access-control-policies)
    * [The Warden](access-control.md#warden)
- [Security Overview](security.md#security-overview)
  - [OAuth 2.0 Security Overview](security.md#oauth-20-security-overview)
    - [Advanced Token Validation (Datastore Security)](security.md#advanced-token-validation-datastore-security)
    - [Access Control Policies](security.md#access-control-policies)
  - [Cryptography](security.md#cryptography)
    - [AES-GCM](security.md#aes-gcm)
    - [RS256](security.md#rs256)
    - [HMAC-SHA256](security.md#hmac-sha256)
    - [BCrypt](security.md#bcrypt)
  - [Access Control](security.md##how-does-access-control-work-with-hydra)
* [Telemetry](telemetry.md)
* How-Tos
  * [Running Hydra in production](how-to/production.md)
  * [Secure the consent app](how-to/consent-app.md)
* [Client Libraries](sdk.md)
  * [Hydra SDK for Go](sdk/go.md)
* [Contribute](contribute.md)
  * [Architecture and Design](contribute.md)
  * [Running Tests](contribute.md)
* [FAQ](faq.md)
  - [How can I control SQL connection limits?](faq.md#how-can-i-control-sql-connection-limits)
  - [Why is the Resource Owner Password Credentials grant not supported?](faq.md#why-is-the-resource-owner-password-credentials-grant-not-supported)
  - [Should I use OAuth2 tokens for authentication?](faq.md#should-i-use-oauth2-tokens-for-authentication)
  - [How to deal with mobile apps?](faq.md#how-to-deal-with-mobile-apps)
  - [How should I run migrations?](faq.md#how-should-i-run-migrations)
  - [What does the installation process look like?](faq.md#what-does-the-installation-process-look-like)
  - [What does a migration process look like?](faq.md#what-does-a-migration-process-look-like)
  - [How can I do this in docker?](faq.md#how-can-i-do-this-in-docker)
  - [Can I set the log level to warn, error, debug, ...?](faq.md#can-i-set-the-log-level-to-warn-error-debug-)
  - [How can I import TLS certificates?](faq.md#how-can-i-import-tls-certificates)
  - [Is there an HTTP API Documentation?](faq.md#is-there-an-http-api-documentation)
  - [How can I disable HTTPS for testing?](faq.md#how-can-i-disable-https-for-testing)
  - [MySQL gives `unsupported Scan, storing driver.Value type []uint8 into type *time.Time`](faq.md#mysql-gives-unsupported-scan-storing-drivervalue-type-uint8-into-type-timetime)
  - [Insufficient Entropy](faq.md#insufficient-entropy)
  - [The docker image exists immediately](faq.md#the-docker-image-exists-immediately)
  - [I get compile errors!](faq.md#i-get-compile-errors)
  - [Is JWT supported?](faq.md#is-jwt-supported)
  - [Refreshing tokens](faq.md#refreshing-tokens)
  - [Revoking tokens & log out](faq.md#revoking-tokens-&-log-out)
  - [Operational Considerations](faq.md#operational-considerations)
    - [Managing Client/Policy Definitions](faq.md#managing-clientpolicy-definitions)
  - [Recovering root client access](faq.md#recovering-root-client-access)
